/*
 * Copyright 2011 Red Hat, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "asn1c_utils.h"

#include "memory.h"

#include <unistd.h>

static int
consume_bytes_file(const void *buffer, size_t size, void *appkey)
{
  FILE *file = (FILE*) appkey;
  int fd = fileno(file);
  return write(fd, buffer, size);
}

static int
consume_bytes_buffer(const void *buffer, size_t size, void *appkey)
{
  char *tmp, **buf = (char**) appkey;
  tmp = realloc(*buf, (*buf ? strlen(*buf) : 0) + size + 1);
  if (!tmp) {
    free(*buf);
    *buf = NULL;
    return -1;
  }

  if (!*buf)
    tmp[0] = '\0';
  *buf = tmp;
  strncat(*buf, buffer, size);
  return 0;
}

bool
int_asn1c_print(asn_TYPE_descriptor_t *type_descriptor, void *ptr, FILE *file)
{
  if (!ptr || !type_descriptor || !file)
    return false;

  return type_descriptor->print_struct(type_descriptor, ptr, 1,
                                       consume_bytes_file, file) >= 0;
}

bool
int_asn1c_dump(asn_TYPE_descriptor_t *type_descriptor, void *ptr, char **buf)
{
  if (!ptr || !type_descriptor || !buf)
    return false;

  *buf = NULL;
  return type_descriptor->print_struct(type_descriptor, ptr, 1,
                                       consume_bytes_buffer, buf) >= 0;
}

bool
int_asn1c_set_flags(BIT_STRING_t *ptr, otpFlags flags)
{
  unsigned int i, size = sizeof(otpFlags) * 8;

  if (!ptr || !ptr->buf || ptr->size < sizeof(otpFlags))
    return false;

  if (size > ptr->size * 8 - ptr->bits_unused)
    size = ptr->size * 8 - ptr->bits_unused;

  for (i = 0; i < size; i++)
    ptr->buf[i / 8] |= flags & (1 << i) ? (1 << (7 - i % 8)) : 0;

  return true;
}

otpFlags
int_asn1c_get_flags(BIT_STRING_t *ptr)
{
  otpFlags flags = 0;
  unsigned int i, size = sizeof(otpFlags) * 8;

  if (!ptr)
    return flags;

  if (size > ptr->size * 8 - ptr->bits_unused)
    size = ptr->size * 8 - ptr->bits_unused;

  for (i = 0; i < size; i++)
    flags |= ptr->buf[i / 8] & (1 << (7 - i % 8)) ? 1 << i : 0;

  return flags;
}
