/*
 * Copyright 2011 Red Hat, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ASN1C_UTILS_H_
#define ASN1C_UTILS_H_

#include "asn1/asn_application.h"
#include "asn1/BIT_STRING.h"
#include "asn1/INTEGER.h"

#include <stdbool.h>

/* OTPFlags ::= KerberosFlags
 *     -- reserved(0),
 *     -- nextOTP(1),
 *     -- combine(2),
 *     -- collect-pin(3),
 *     -- do-not-collect-pin(4),
 *     -- must-encrypt-nonce (5),
 *     -- separate-pin-required (6),
 *     -- check-digit (7)
 */
typedef enum otpFlags_ {
  otpFlagNone = 0,
  otpFlagReserved = 1 << 0,
  otpFlagNextOTP = 1 << 1,
  otpFlagCombine = 1 << 2,
  otpFlagCollectPIN = 1 << 3,
  otpFlagDoNotCollectPIN = 1 << 4,
  otpFlagMustEncryptNonce = 1 << 5,
  otpFlagSeparatePINRequired = 1 << 6,
  otpFlagCheckDigit = 1 << 7,
  _otpFlagMAX = otpFlagCheckDigit
} otpFlags;

bool
int_asn1c_print(asn_TYPE_descriptor_t *type_descriptor, void *ptr, FILE *file);

bool
int_asn1c_dump(asn_TYPE_descriptor_t *type_descriptor, void *ptr, char **buf);

bool
int_asn1c_set_flags(BIT_STRING_t *ptr, otpFlags flags);

otpFlags
int_asn1c_get_flags(BIT_STRING_t *ptr);

#endif /* ASN1C_UTILS_H_ */
